##############################################################################
use strict;

#use FindBin qw($Bin);
use File::Copy;
use POSIX qw(strftime);
use Data::Dumper;
use vars qw($CONFIG $CONST @ERR);

sub ReadConfFile($$;$)
{
  my ($Data,$ConfPath,$ValMap) = @_;
  my %valmap;

  if(defined $ValMap)
  {
    my ($key, $value);
    while( ($key, $value) = each(%{$ValMap}) )
    {
      $value = $value;
      $valmap{$value} = $key;
    }
  }

  if( open( CFG, "<$ConfPath" ) )
  {
    while(<CFG>)
    {
      chomp;
      next if /(^\s*\#)|(^\s*$)/;

      if( /^\s*(\S+)(\s+(.*))?$/ )
      {
        my ($name, $val) = ($1, $3);
        if( !defined($ValMap) || defined($valmap{$name}) )
        {
          $val =~ s/\s+$//;
          $val =~ s/^\s+//;
          $name = $valmap{$name} if defined($ValMap);
          $Data->{$name} = ( defined($val) && $val ne '' )? $val: undef;
        }
      }
    }
    close CFG;
    return 1;
  }

  return undef;
}


sub WriteConfFile($$;$)
{
  my ($Data,$ConfPath,$ValMap) = @_;
  my %data;

  if(defined $ValMap)
  {
    foreach my $key (keys %{$Data})
    {
      $data{ $ValMap->{$key} } = $Data->{$key} if( defined( $ValMap->{$key} ) );
    }
  }
  else
  {
    %data = %{$Data};
  }

  my $TempPath = $CONFIG->{FilterTmpConfigPath} . "/.write_conf_file.$$.tmp";
  my $mark = 1;
  my $MARKER = '#### Updated by Control Center at ';
  my $TIMEFORMAT = '%Y-%m-%d %H:%M:%S ####';

  umask $CONST->{'umask'} if defined($CONST) && defined($CONST->{'umask'});

  unless( open( OUT, ">$TempPath") )
  {
    #warn "Can't open file $TempPath: $!\n";
    push @ERR, "Can't open file $TempPath: $!";
    return undef;
  }

  if( open( IN, "<$ConfPath" ) )
  {
    while(<IN>)
    {
      chomp;

      if( /^\Q$MARKER/o )
      {
        print OUT $MARKER, strftime($TIMEFORMAT,localtime), "\n" if $mark;
        $mark = undef;
        next;
      }

      if( /(^\s*\#)|(^\s*$)/ )
      {
        print OUT "$_\n";
        next;
      }

      if( /^\s*(\S+)(\s+(.*))?$/ )
      {
        my ($name, $val) = ($1, $3);
        if( exists( $data{$name} ) )
        {
          next unless( defined $data{$name} ); # Erase value
          print OUT $name, " ",  $data{$name}, "\n";
          $data{$name} = undef;
        }
        else
        {
          print OUT "$_\n";
        }
      }
      else
      {
        print OUT "$_\n";
      }
    }

    close IN;
  }

  if( $mark )
  {
    print OUT "\n", $MARKER, strftime($TIMEFORMAT,localtime), "\n" if $mark;
    $mark = undef;
  }

  foreach ( sort keys(%data) )
  {
    next unless defined $data{$_};
    print OUT "$_ $data{$_}\n";
  }

  close OUT;

  unless( move( $TempPath, $ConfPath ) )
  {
    unlink $TempPath;
#    warn "Can't move file $TempPath to $ConfPath: $!\n";
    push @ERR, "Can't move file $TempPath to $ConfPath: $!";
    return undef;
  }

  return 1;
}


sub ReadIniFile($$;$)
{
  my ($Data,$IniPath,$ValMap) = @_;
  my %valmap;

  if(defined $ValMap)
  {
    my ($key, $value);
    while( ($key, $value) = each(%{$ValMap}) )
    {
#      $value = lc($value);
      $valmap{$value} = $key;
    }
  }

  if( open( INI, "<$IniPath" ) )
  {
    my $section = undef;

    while(<INI>)
    {
      chomp;
      next if /(^\s*(\;|\#))|(^\s*$)/;

      if( /^\s*\[([^]]+)\]/ )
      {
#        $section = lc($1);
        $section = $1;
        $section =~ s/^\s+//;
        $section =~ s/\s+$//;
        $section .= '/';

        next;
      }

      if( /^\s*([^=]+)\s*\=\s*(.*)$/ )
      {
#        my ($name, $val) = (lc($1), $2);
        my ($name, $val) = ($1, $2);
        $name =~ s/^\s+//;
        $name =~ s/\s+$//;
        $name = $section . $name if defined $section;

        if( !defined($ValMap) || defined($valmap{$name}) )
        {
          $val =~ s/\s+$//;
          $val =~ s/^\s+//;
          $name = $valmap{$name} if defined($ValMap);
          $Data->{$name} = ( defined($val) && $val ne '' )? $val: undef;
        }
      }
    }
    close INI;
    return 1;
  }

  return undef;
}


sub WriteIniFile($$;$)
{
  my ($Data,$IniPath,$ValMap) = @_;
  my %data;

  if(defined $ValMap)
  {
    foreach my $key (keys %{$Data})
    {
#      $data{ lc($ValMap->{$key}) } = $Data->{$key} if( defined( $ValMap->{$key} ) );
      $data{$ValMap->{$key}} = $Data->{$key} if( defined( $ValMap->{$key} ) );
    }
  }
  else
  {
    %data = %{$Data};
  }

  my $sections = {};
  my ($key,$section,$item,$val);
  while( ($item, $val) = each %data )
  {
    next unless defined $item;

    if( $item =~ m#^([^/\\]+)/(.+)$# )
    {
      $section = $1;
      $key = $2;
    }
    else
    {
      $section = '';
      $key = $item;
      $key =~ s#([\\/\[\]])##g;
    }

    $key = '?' unless defined $key;
    $section =~ s#[\[\]]##g if defined $section;

    $sections->{$section} = {} unless defined $sections->{$section};
    $sections->{$section}{$key} = $val;
  }
  %data = ();

  my $TempPath = $CONFIG->{FilterTmpConfigPath} . "/.write_ini_file.$$.tmp";
  umask $CONST->{'umask'} if defined($CONST) && defined($CONST->{'umask'});

  unless( open( OUT, ">$TempPath") )
  {
#    warn "Can't open file $TempPath: $!\n";
    push @ERR, "Can't open file $TempPath: $!";
    return undef;
  }

  $section = '';
  if( open( IN, "<$IniPath" ) )
  {
    while(<IN>)
    {
      chomp;

      next if /^\s*$/;

      if( /(^\s*(\#|\;))/ )
      {
        print OUT "$_\n";
        next;
      }

      if( /^\s*\[([^\]]+)\]/ )
      {
#        my $newsection = lc($1);
        my $newsection = $1;
        $newsection =~ s/^\s+//;
        $newsection =~ s/\s+$//;

        if( defined $sections->{$section} )
        {
          foreach $key (sort keys %{$sections->{$section}})
          {
            $val = $sections->{$section}{$key};
            next unless defined $val;
            print OUT "$key=$val\n";
          }

          delete $sections->{$section};
        }

        print OUT "\n[$newsection]\n";
        $section = $newsection;
        next;
      }

      if( /^\s*([^=]+)\s*\=/ )
      {
#        $key = lc($1);
        $key = $1;
        $key =~ s/^\s+//;
        $key =~ s/\s+$//;

        if( defined($sections->{$section}) && exists($sections->{$section}{$key}) )
        {
          print OUT "$key=", $sections->{$section}{$key}, "\n" if defined $sections->{$section}{$key};
          $sections->{$section}{$key} = undef;
        }
        else
        {
          print OUT "$_\n";
        }
      }
    }
    close IN;
  }

  if( defined $sections->{$section} )
  {
    foreach $key (sort keys %{$sections->{$section}})
    {
      $val = $sections->{$section}{$key};
      next unless defined $val;
      print OUT "$key=$val\n";
    }

    delete $sections->{$section};
  }

  foreach $section (sort keys(%{$sections}))
  {
    print OUT "\n[$section]\n";

    foreach $key (sort keys %{$sections->{$section}})
    {
      $val = $sections->{$section}{$key};
      $sections->{$section}{$key} = undef;
      next unless defined $val;
      print OUT "$key=$val\n";
    }
  }

  close OUT;

  unless( move( $TempPath, $IniPath ) )
  {
    unlink $TempPath;
#    warn "Can't move file $TempPath to $IniPath: $!\n";
    push @ERR, "Can't move file $TempPath to $IniPath: $!";
    return undef;
  }

  return 1;
}

1;
__END__

# Debug code

my $path = '/usr/local/ap-mailfilter3/etc/kasupdater.conf';
$CONFIG = {};
$CONFIG->{FilterTmpConfigPath} = '/usr/local/ap-mailfilter3/conf/tmp';
my $conf = {};
my $map = {
            Keepup2DateBasePath => 'path/basespath',
            Keepup2DateLicensePath => 'path/licensepath',
            Keepup2DateConnectTimeout => 'updater.options/connecttimeout',
            Keepup2DateBackupPath => 'updater.path/backuppath',
            aaa => 'path/aaa',
            bbb => 'updater.options/bbb',
            ccc => 'updater.path/ccc',
            ddd => 'ddd/ddd',
            eee => '/eee',
            fff => 'fff/',
            ggg => 'ggg',
          };

ReadIniFile( $conf, $path, $map );

$conf->{aaa} = 1;
$conf->{bbb} = 2;
$conf->{ccc} = 3;
$conf->{ddd} = 4;
$conf->{eee} = 5;
$conf->{fff} = 6;
$conf->{ggg} = 7;
print Dumper($conf);

WriteIniFile( $conf, $path, $map );
