#!/usr/local/ap-mailfilter3/bin/perl -w
#######################################################################
# Default paths and constants

use strict;
use POSIX qw(strftime mktime);
use Data::Dumper;
use File::Copy;
use Time::Local;
use Sys::Hostname;

use vars qw($CONST $CONFIG @ERR);

BEGIN {
  require 'const.pl';
  require 'config.pl';
  require 'conffiles.pl';
  require 'utils.pl';
};


sub ps()
{
  my @out = `$CONST->{bin}{ps}`;
  my $ret = $?>>8;
  if( $ret )
  {
#    warn "\"$CONST->{bin}{ps}\" return $ret\n";
    push @ERR, "\"$CONST->{bin}{ps}\" return $ret";
    return ();
  }

  return @out;
}


sub CheckByPIDFile($)
{
  my ($PIDPath) = @_;

  return undef unless( -e $PIDPath && -s _ && -r _ );
  unless( open( PIDFILE, "<$PIDPath" ) )
  {
#    warn "Can't open PID file $PIDPath: $!\n";
    push @ERR, "Can't open PID file $PIDPath: $!";
    return undef;
  }

  my $PID = <PIDFILE>;
  close(PIDFILE);
  $PID = int($PID);

  my $PIDProc = undef;
  foreach (ps())
  {
    if( /^\s*(\d+)\s+/ )
    {
      if( $1 == $PID )
      {
        $PIDProc = $PID;
        last;
      }
    }
  }

  return $PIDProc;
}


sub IsApProcessServerRunning()
{
  # return pid
  return CheckByPIDFile( $CONFIG->{ServerPIDFile} );
}


sub GetApFilters()
{
  # returt ap-mailfilter processes pids array
  my @lines = grep{ /[\[ ]ap-mailfilter/ } ps();

  foreach (@lines)
  {
    chomp;
    s/^\s*(\d+)\s+.*$/int($1)/e;
  }
  return @lines;
}


sub IsApFiltersExist()
{
  return -x $CONFIG->{FilterPath};
}


sub GetSpfds()
{
  # return ap-spfd processes pids array
  my @lines = grep{ /[\/\[ ]ap-spfd/ } ps();

  foreach (@lines)
  {
    chomp;
    s/^\s*(\d+)\s+.*$/int($1)/e;
  }
  return @lines;
}


sub IsSpfDaemonRunning()
{
  # return pid
  return CheckByPIDFile( $CONFIG->{SpfDaemonPIDFile} );
}


sub GetLastUpdateInfo()
{
  # Return array: (compile_time, update_id)
  my $mainset_info_name = $CONFIG->{FilterBaseIDPath};
  my ($update_time,$update_id) = (0,undef);

  unless( open( INFO, "<$CONFIG->{FilterBaseIDPath}" ) )
  {
#    warn "Can't open $mainset_info_name: $!\n";
    push @ERR, "Can't open $mainset_info_name: $!";
  }
  else
  {
    while( <INFO> )
    {
      chomp;
      next if /^\s*$/;
      if( /^\s*compiled:\s*(\d+)-(\d+)-(\d+)\s+(\d+):(\d+):(\d+)\s*/i )
      {
        my ($y,$m,$d,$H,$M,$S) = ($1,$2,$3,$4,$5,$6);
        $update_time = timelocal($S,$M,$H,$d,$m-1,$y-1900);
      }
      else
      {
        if( defined $update_id )
        {
          $update_id .= "\n$_";
        }
        else
        {
          $update_id = $_;
        }
      }
    }
    close INFO;
  }

  $update_time = (stat($mainset_info_name))[9] unless( defined $update_time );

  return ($update_time,$update_id);
}


sub IsLicenseServiceRunning()
{
  # return pid
  return CheckByPIDFile( $CONFIG->{KasLicensePIDFile} );
}


sub GetLicenseInfo()
{
  # return reference to hash with lic info
  my $cmd = $CONFIG->{LicInfoPath};

  unless( open(INFO,"$cmd|") )
  {
    push @ERR, "Can't execute \"$cmd\": $!";
    return undef;
  }

  my %res = ( product=>undef, valid=>0, license=>undef, expires=>0, key=>[] );

  while(<INFO>)
  {
    chomp;
    next unless /^\*/;
    if( /^\*product:\s*(.*?)\s*$/i )
    {
      $res{product} = $1;
    }
    elsif( /^\*licen[sc]e:\s*(.*?)\s*$/i )
    {
      my ($lic, $vol) = split( /\s*:\s*/, $1, 2 );
      $res{license} = $lic;
      $res{license} .= " $vol" if($vol);
    }
    elsif( /^\*valid till:\s*(\d+)\.(\d+)\.(\d+)/i )
    {
      my ($d,$m,$y) = ($1,$2,$3);
      if($y)
      {
        eval { $res{valid} = timelocal( 0, 0, 12, $d, $m-1, $y-1900 ); };
      }
    }
    elsif( /^\*expires:\s*(.*)$/i )
    {
      $res{expires} = int($1);
    }
    elsif( /^\*(Active|Additional) key:\s*(.*?)\s*$/i )
    {
      my $ind = $1 eq "Active" ? 0 : 1;
      my ($file,$serial,$version,$appid,$productid,$type,$amount,$datestr) = split( /\s*:\s*/, $2, 8 );
      my $valid = 0;
      if( $datestr =~ /(\d+)\.(\d+)\.(\d+)/ )
      {
        my ($d,$m,$y) = ($1,$2,$3);
        if($y)
        {
          eval { $valid = timelocal( 0, 0, 12, $d, $m-1, $y-1900 ); };
        }
      }

      my %key = ( file=>$file, serial=>$serial, version=>$version,
                  type=>$type, amount=>int($amount), valid=>$valid );
      @{$res{key}}[$ind] = \%key;
    }
  }
  close(INFO);
  return \%res;
}

sub IsHttpDaemonRunning()
{
  # return pid
  my $flag = $CONFIG->{MonitoringHttpd}!~/no/i;
  if($flag)
  {
    return CheckByPIDFile( $CONFIG->{HttpDaemonPIDFile} );
  }
  else
  {
    return -1;
  }
}

sub IsKasMilterRunning()
{
  # return pid
  my $flag = $CONFIG->{MonitoringKasMilter}!~/no/i;
  if($flag)
  {
    return CheckByPIDFile( $CONFIG->{KasMilterPIDFile} );
  }
  else
  {
    return -1;
  }
}

sub GetSystemInfo()
{
  # struct with info about system

  my %res = ( hostname=>FullHostname() );
  $res{hostname} = 'unknown' unless defined($res{hostname});
  $res{system_id} = `$CONST->{bin}{uname}`;
  $res{filter_id} = `$CONFIG->{FilterPath} --print-version`;

  $res{filter_id} =~ s/^SMTP-Filter Version //i;
  $res{filter_id} =~ s/ version,/,/i;
  $res{filter_id} =~ s/(built at.*?)by\s+\S+/$1/i;

  if( open(UPTIME,"$CONST->{bin}{uptime}|") )
  {
    while(<UPTIME>)
    {
      chomp;
      if( /load averages?:\s+(\d+[,.]\d+),?\s+(\d+[,.]\d+),?\s+(\d+[,.]\d+)/i )
      {
        my @load = ($1, $2, $3);
        foreach (@load) { s/\,/\./g; }
        $res{load_average} = \@load;
      }
    }
    close(UPTIME);

    if( $res{load_average} )
    {
      my $load5 = ${$res{load_average}}[0];
      my $LA_level;
      if( $load5 < $CONST->{etc}{load_average}{warning} )
      {
        $res{load_average_level} = 'notice';
      }
      elsif( $load5 < $CONST->{etc}{load_average}{error} )
      {
        $res{load_average_level} = 'warning';
      }
      else
      {
        $res{load_average_level} = 'error';
      }
    }
  }

  return \%res;
}


sub LoadEvents()
{
  my $path = $CONFIG->{CCEventsPath};
  my %events;

  if( open( FILE, "<$path" ) )
  {
    # Format:
    # event_name:check_time:count:start_time:mail_time:value

    while(<FILE>)
    {
      chomp;
      if( /^([^:]+):(\d+):(\d+):(\d+):(\d+):(.*)$/ )
      {
        my ($event,$checktime,$count,$starttime,$mailtime,$value) = ($1,int($2),int($3),int($4),int($5),$6);
        $events{$event} = { check_time=>$checktime, mail_time=>$mailtime, start_time=>$starttime, count=>$count, value=>$value };
        $events{$event}{value} = undef if $events{$event}{value} eq '';
      }
    }
    close(FILE);
  }
  else
  {
#    warn "Can't open event file $path: $!\n";
  }

  return \%events;
}


sub SaveEvents($)
{
  my ($events) = @_;
  return undef unless defined $events;

  my $path = $CONFIG->{CCEventsPath};
  my $tmppath = $path.".$$.tmp";

  if( open( FILE, ">$tmppath" ) )
  {
    # Format:
    # event_name:check_time:count:start_time:mail_time:value

    foreach my $event (sort keys(%{$events}))
    {
      next unless $event;
      $events->{$event}{count} = 0 unless( defined($events->{$event}{count}) );
      $events->{$event}{check_time} = 0 unless $events->{$event}{check_time};
      $events->{$event}{start_time} = 0 unless $events->{$event}{start_time};
      $events->{$event}{mail_time} = 0 unless $events->{$event}{mail_time};

      if( $events->{$event}{count} )
      {
        $events->{$event}{start_time} = time() unless $events->{$event}{start_time};
      }
      else
      {
        $events->{$event}{start_time} = $events->{$event}{mail_time} = 0;
      }

      $events->{$event}{value} = '' unless defined $events->{$event}{value};
      $events->{$event}{value} =~ s/^\s+//;
      $events->{$event}{value} =~ s/\s+$//;
      $events->{$event}{value} =~ s/[\n\r\t]+/ /g;
      $events->{$event}{value} =~ s/\s+/ /g;

      print FILE "$event:",int($events->{$event}{check_time}),":",int($events->{$event}{count}),":",int($events->{$event}{start_time}),":",int($events->{$event}{mail_time}),":$events->{$event}{value}\n";
    }
    close(FILE);

    if( move $tmppath, $path )
    {
      return 1;
    }
    else
    {
#      warn "Can't move $tmppath to $path: $!\n";
      push @ERR, "Can't move $tmppath to $path: $!";
    }
  }
  else
  {
#    warn "Can't open file $tmppath: $!\n";
    push @ERR, "Can't open file $tmppath: $!";
  }

  return undef;
}


sub event_verify_defined($)
{
  my ($ref) = @_;

  if( defined($ref->{value}) && $ref->{value} ne '' )
  {
    $ref->{start_time} = $ref->{count} = 0;
  }
  else
  {
    $ref->{start_time} = time() unless( $ref->{count} );
    $ref->{count}++;
  }
}


sub event_verify_true($)
{
  my ($ref) = @_;

  if( $ref->{value} )
  {
    $ref->{start_time} = $ref->{count} = 0;
  }
  else
  {
    $ref->{start_time} = time() unless( $ref->{count} );
    $ref->{count}++;
  }
}


sub event_verify($$)
{
  my ($ref,$func) = @_;

  if( &$func($ref->{value}) )
  {
    $ref->{start_time} = $ref->{count} = 0;
  }
  else
  {
    $ref->{start_time} = time() unless( $ref->{count} );
    $ref->{count}++;
  }
}

sub get_midnight($)
{
  my ($time) = @_;
  my @time = localtime($time);
  $time[0] = $time[1] = $time[2] = 0;
  return timelocal(@time);
}


sub GetEvents(%)
{
  my (%param) = @_;
  my %events;
  my $modified = 0;
  my $INFO = $CONST->{etc}{info_label};

  open(OLDERR,'>&STDERR');
  open(STDERR,'>/dev/null');

  if( $param{all} )
  {
    foreach (qw(system_info apfilters approc_server spfd licdaemon httpdaemon kasmilter crontab update license midnight)) { $param{$_} = 1; }
    $param{all} = undef;
  }
  $param{approc_server} = 1 if( $param{apfilters} );

  {
    my $ref_data = LoadEvents();
    %events = %{$ref_data} if $ref_data;
  }

  if( $param{system_info} )
  {
    my $time = time();
    if( $param{nocache} || ($events{system_info}{check_time} > $time)
                        || ($events{system_info}{check_time} + $CONST->{cachetime}{system_info} <= $time) )
    {
      $modified = 1;
      $events{system_info}{check_time} = $time;

      my $sys_info = GetSystemInfo();
      if( $sys_info )
      {
        foreach (qw(load_average load_average_level hostname filter_id os))
        {
          $events{$_} = {} unless defined $events{$_};
          $events{$_}{check_time} = $time;
        }

        if( $sys_info->{load_average_level} )
        {
          $events{load_average}{value} = ${$sys_info->{load_average}}[0];
          $events{load_average_level}{value} = $sys_info->{load_average_level};

          if( $events{load_average_level}{value} eq 'error' )
          {
            $events{load_average_level}{start_time} = $time unless( $events{load_average_level}{count} );
            $events{load_average_level}{count}++;
          }
          else
          {
            $events{load_average_level}{start_time} = $events{load_average_level}{count} = 0;
          }
        }
        else
        {
          $events{load_average}{value} = $events{load_average_level}{value} = undef;
        }

        $events{filter_id}{value} = $sys_info->{filter_id}; event_verify_defined($events{filter_id});
        $events{hostname}{value} = $sys_info->{hostname};
        $events{os}{value} = $sys_info->{system_id};
      }
    }

    my $LA_message = '';
    $LA_message = " - $LANG->{error}{load_average_too_high}" if( $events{load_average_level}{value} && $events{load_average_level}{value} eq 'error' );

    $events{load_average}{message} = $events{load_average_level}{value} ? { alert=>$events{load_average_level}{value}, text=>"$events{load_average}{value}$LA_message" } : {};
    $events{load_average_level}{message} = $events{load_average}{message};
    $events{filter_id}{message}    = $events{filter_id}{value}? { alert=>$INFO, text=>$events{filter_id}{value} } : { alert=>'error', text=>$LANG->{error}{not_found} };
    $events{hostname}{message}     = $events{hostname}{value}? { alert=>$INFO, text=>$events{hostname}{value} }: { alert=>'warning', text=>'localhost' };
    $events{os}{message}           = $events{os}{value}? { alert=>$INFO, text=>$events{os}{value} }: { alert=>'warning', text=>$LANG->{error}{no_information} };
  }

  if( $param{approc_server} )
  {
    my $time = time();
    if( $param{nocache} || ($events{approc_server}{check_time} > $time)
                        || ($events{approc_server}{check_time} + $CONST->{cachetime}{approc_server} <= $time) )
    {
      $modified = 1;
      $events{approc_server} = {} unless defined $events{approc_server};
      $events{approc_server}{check_time} = $time;

      my $proc_server_pid = IsApProcessServerRunning();
      $events{approc_server}{value} = $proc_server_pid; event_verify_true($events{approc_server});
    }

    $events{approc_server}{message} = $events{approc_server}{value} ? 
                { alert=>'notice', text=>"$LANG->{static}{status_ok}, pid=$events{approc_server}{value}" } :
                { alert=>'error', text=>$LANG->{error}{proc_not_running} };
  }

  if( $param{apfilters} )
  {
    my $time = time();
    if( $param{nocache} || ($events{apfilters}{check_time} > $time)
                        || ($events{apfilters}{check_time} + $CONST->{cachetime}{apfilters} <= $time) )
    {
      $modified = 1;
      $events{apfilters}{check_time} = $time;

      my @filter_pid = GetApFilters();
      my $filter_exist = IsApFiltersExist();

      foreach (qw(filter_count filter_exist))
      {
        $events{$_} = {} unless defined $events{$_};
        $events{$_}{check_time} = $time;
      }

      $events{filter_exist}{value} = $filter_exist; event_verify_true($events{filter_exist});
      $events{filter_count}{value} = scalar(@filter_pid);
    }

    unless($events{filter_exist}{value})
    {
      $events{apfilters}{message} = { alert=>'error', text=>$LANG->{error}{exec_not_found} };
      $events{apfilters}{value} = 0;
    }
    elsif( $events{approc_server}{message}{alert} eq 'error' )
    {
      $events{apfilters}{message} = { alert=>'error', text=>$LANG->{error}{proc_not_running} };
      $events{apfilters}{value} = 0;
    }
    else
    {
      my $text = "$LANG->{static}{status_ok}, $LANG->{static}{status_proc_running}: $events{filter_count}{value}";
      $events{apfilters}{message} = { alert=>'notice', text=>$text };
      $events{apfilters}{value} = 1;
    }
    event_verify_true($events{apfilters});
  }

  if( $param{spfd} )
  {
    my $time = time();
    if( $param{nocache} || ($events{spfd_exist}{check_time} > $time)
                        || ($events{spfd_exist}{check_time} + $CONST->{cachetime}{spfd} <= $time) )
    {
      $modified = 1;
      $events{spfd_exist} = {} unless defined $events{spfd_exist};
      $events{spfd_exist}{check_time} = $time;

      $events{spfd_count} = {} unless defined $events{spfd_count};
      $events{spfd_count}{check_time} = $time;

      my @spfd_pid = GetSpfds();
      my $spfd_exist = IsSpfDaemonRunning();

      $events{spfd_exist}{value} = $spfd_exist;
      event_verify_true($events{spfd_exist});
      
      $events{spfd_count}{value} = scalar(@spfd_pid);
    }

    unless($events{spfd_exist}{value})
    {
      $events{spfd_exist}{message} = { alert=>'error', text=>$LANG->{error}{proc_not_running} };
    }
    else
    {
      my $count = $events{spfd_count}{value};
      $count-- if($count);

      my $text = "$LANG->{static}{status_ok}, $LANG->{static}{status_proc_running}: $count";
      $events{spfd_exist}{message} = { alert=>'notice', text=>$text };
    }
  }

  if( $param{licdaemon} )
  {
    my $time = time();
    if( $param{nocache} || ($events{licdaemon}{check_time} > $time)
                        || ($events{licdaemon}{check_time} + $CONST->{cachetime}{licdaemon} <= $time) )
    {
      $modified = 1;
      $events{licdaemon}{check_time} = $time;
      $events{licdaemon} = {} unless defined $events{licdaemon};

      my $lic_pid = IsLicenseServiceRunning();
      $events{licdaemon}{value} = $lic_pid; event_verify_true($events{licdaemon});
    }

    $events{licdaemon}{message} = $events{licdaemon}{value} ? { alert=>'notice', text=>"$LANG->{static}{status_ok}, pid=$events{licdaemon}{value}" } : { alert=>'error', text=>$LANG->{error}{proc_not_running} };
  }

  if( $param{httpdaemon} )
  {
    my $time = time();
    if( $param{nocache} || ($events{httpdaemon}{check_time} > $time)
                        || ($events{httpdaemon}{check_time} + $CONST->{cachetime}{httpdaemon} <= $time) )
    {
      $modified = 1;
      $events{httpdaemon} = {} unless defined $events{httpdaemon};
      $events{httpdaemon}{check_time} = $time;

      my $lic_pid = IsHttpDaemonRunning();
      $events{httpdaemon}{value} = $lic_pid; event_verify_true($events{httpdaemon});
    }

    $events{httpdaemon}{message} = $events{httpdaemon}{value} ?
          { alert=>'notice', text=>"$LANG->{static}{status_ok}, pid=$events{httpdaemon}{value}" } :
          { alert=>'error', text=>$LANG->{error}{proc_not_running} };
  }

  if( $param{kasmilter} )
  {
    my $time = time();
    if( $param{nocache} || ($events{kasmilter}{check_time} > $time)
                        || ($events{kasmilter}{check_time} + $CONST->{cachetime}{kasmilter} <= $time) )
    {
      $modified = 1;
      $events{kasmilter} = {} unless defined $events{kasmilter};
      $events{kasmilter}{check_time} = $time;

      my $lic_pid = IsKasMilterRunning();
      $events{kasmilter}{value} = $lic_pid; event_verify_true($events{kasmilter});
    }

    $events{kasmilter}{message} = $events{kasmilter}{value} ?
          { alert=>'notice', text=>"$LANG->{static}{status_ok}, pid=$events{kasmilter}{value}" } :
          { alert=>'error', text=>$LANG->{error}{proc_not_running} };
  }

  if( $param{crontab} )
  {
    my $time = time();
    if( $param{nocache} || ($events{crontab}{check_time} > $time)
                        || ($events{crontab}{check_time} + $CONST->{cachetime}{crontab} <= $time) )
    {
      $modified = 1;
      $events{crontab}{check_time} = $time;

      my $cron_info = GetCrontabScriptInfo();
      if( $cron_info )
      {
        foreach (qw(dologs logrotate updater dograph monitoring udsrtts))
        {
          $events{"crontab_$_"} = {} unless defined $events{"crontab_$_"};
          $events{"crontab_$_"}{check_time} = $time;
          $events{"crontab_$_"}{value} = $cron_info->{$_};
          event_verify( $events{"crontab_$_"}, sub { my $r = shift @_; return $r>0; } );
        }
      }
    }

    if( defined($events{crontab_updater}{value}) )
    {
      if( $events{crontab_updater}{value} < 0 )
      {
        $events{crontab_updater}{message} = { alert=>'error', text=>$LANG->{error}{mon_updater_not_found} };
      }
      elsif( $events{crontab_updater}{value} )
      {
        $events{crontab_updater}{message} = { alert=>'notice', text=>$LANG->{error}{mon_updater_enabled} };
      }
      else
      {
        $events{crontab_updater}{message} = { alert=>'warning', text=>$LANG->{error}{mon_updater_not_in_crontab} };
      }
    }
    else
    {
      $events{crontab_updater}{message} = { alert=>'error', text=>$LANG->{error}{crontab_get_fault} };
    }

    my $monitoring_level = 1;
    my $not_found = "";
    my $not_in_crontab = "";
    foreach (qw(dologs dograph logrotate monitoring udsrtts))
    {
      next unless defined $events{"crontab_$_"}{value};
      $monitoring_level = $events{"crontab_$_"}{value} if( $monitoring_level > $events{"crontab_$_"}{value} );
      if($events{"crontab_$_"}{value} < 0)
      {
        $not_found .= ", " if($not_found);
        $not_found .= $LANG->{scriptnames}{$_};
      }
      elsif($events{"crontab_$_"}{value} == 0)
      {
        $not_in_crontab .= ", " if($not_in_crontab);
        $not_in_crontab .= $LANG->{scriptnames}{$_};
      }
    }

    $events{crontab_status}{check_time} = $time;
    $events{crontab_status}{value} = $monitoring_level;
    if( $monitoring_level < 0 )
    {
      my $text = "$LANG->{error}{scripts_not_found}: $not_found";
      $events{crontab_status}{message} = { alert=>'error', text=>$text };
    }
    elsif( $monitoring_level )
    {
      $events{crontab_status}{message} = { alert=>'notice', text=>$LANG->{static}{status_ok} };
    }
    else
    {
      my $text = "$LANG->{error}{scripts_not_in_crontab}: $not_in_crontab";
      $events{crontab_status}{message} = { alert=>'error', text=>$text };
    }
    event_verify( $events{crontab_status}, sub { my $r = shift @_; return $r>0; } );
  }

  if( $param{update} )
  {
    my $time = time();
    if( $param{nocache} || ($events{update}{check_time} > $time)
                        || ($events{update}{check_time} + $CONST->{cachetime}{update} <= $time) )
    {
      $modified = 1;
      $events{update}{check_time} = $time;

      foreach (qw(update_id update_time))
      {
        $events{$_} = {} unless defined $events{$_};
        $events{$_}{check_time} = $time;
      }

      my @update = GetLastUpdateInfo();
      if( @update )
      {
        my ($updtime,$id) = @update;
        $events{update_time}{value} = $updtime;
        $events{update_id}{value}   = $id;
      }
      else
      {
        $events{update_time}{value} = 0;
        $events{update_id}{value} = undef;
      }
    }

    unless( $events{update_time}{value} )
    {
      $events{update_time}{message} = { alert=>'error', text=>$LANG->{error}{no_information} };
    }
    else
    {
      $events{update_time}{message} = { text=>strftime($LANG->{timefmt}{datetime_short},localtime($events{update_time}{value})) };
      my $old = (time() - $events{update_time}{value}) / 3600.;

      if( $old < $CONST->{etc}{update_time}{warning} )
      {
        $events{update_time}{message}{alert} = 'notice';
      }
      elsif( $old < $CONST->{etc}{update_time}{error} )
      {
        $events{update_time}{message}{alert} = 'warning';
        $events{update_time}{message}{text} .= " - $LANG->{error}{base_too_old}";
      }
      else
      {
        $events{update_time}{message}{alert} = 'error';
        $events{update_time}{message}{text} .= " - $LANG->{error}{base_too_old}";
      }
    }
    $events{update_id}{message} = $events{update_id}{value}? { alert=>$INFO, text=>$events{update_id}{value} }: { alert=>'error', text=>$LANG->{error}{no_information} };

    event_verify( $events{update_time}, sub{ $events{update_time}{message}{alert} eq 'notice' } );
    event_verify( $events{update_id},   sub{ $events{update_id}{message}{alert} eq $INFO } );
  }

  if( $param{license} )
  {
    my $time = time();
    if( $param{nocache} || ($events{license}{check_time} > $time)
                        || ($events{license}{check_time} + $CONST->{cachetime}{license} <= $time) )
    {
      $modified = 1;
      $events{license}{check_time} = $time;

      foreach (qw(lic_product lic_type lic_valid lic_key lic_expires))
      {
        $events{$_} = {} unless defined $events{$_};
        $events{$_}{check_time} = $time;
      }

      my $lic = GetLicenseInfo();
      if( $lic )
      {
        $events{lic_product}{value} = $lic->{product};
        $events{lic_type}{value}    = $lic->{license};
        $events{lic_valid}{value}   = $events{lic_type}{value} =~ /^INVALID/ ? 0 : $lic->{valid};
        $events{lic_key}{ext}       = $lic->{key};
        $events{lic_expires}{value} = $lic->{expires};
      }
      else
      {
        $events{lic_product}{value} = $events{lic_type}{value} = undef;
        $events{lic_valid}{value} = 0;
      }
      event_verify_true($events{lic_valid});
      event_verify_defined($events{lic_product});
      event_verify_defined($events{lic_type});
    }

    $events{lic_product}{message} = $events{lic_product}{value}? { alert=>$INFO, text=>$events{lic_product}{value} }: { alert=>'error', text=>$LANG->{error}{no_information} };
    
    $events{lic_type}{message}    = $events{lic_type}{value}?    { alert=>$INFO, text=>$events{lic_type}{value} }   : { alert=>'error', text=>$LANG->{error}{no_keys_found} };
    if($events{lic_type}{value} =~ /^INVALID/)
    {
      $events{lic_type}{message}{alert} = 'error';
      event_verify( $events{lic_type}, sub{ $events{lic_type}{message}{alert} eq 'notice' } );

      $events{lic_valid}{message}{alert} = 'error';
    }
    elsif( $events{lic_valid}{value} )
    {
      $events{lic_valid}{message} = {};
      $events{lic_valid}{message}{text} = FormatDateTime($LANG->{tm_format}{date}, localtime($events{lic_valid}{value}));

      if( $events{lic_expires}{value} >= $CONST->{etc}{license_time}{warning} )
      {
        $events{lic_valid}{message}{alert} = 'notice';
        $events{lic_valid}{message}{text} .= " " . sprintf($LANG->{static}{lic_expires_in}, $events{lic_expires}{value});
      }
      elsif( $events{lic_expires}{value} >= $CONST->{etc}{license_time}{error} )
      {
        $events{lic_valid}{message}{alert} = 'warning';
        $events{lic_valid}{message}{text} .= " " . sprintf($LANG->{static}{lic_expires_in}, $events{lic_expires}{value});
      }
      else
      {
        $events{lic_valid}{message}{alert} = 'error';
        $events{lic_valid}{message}{text} .= " $LANG->{error}{licvalid_outdated}";
      }
    }
    else
    {
      $events{lic_valid}{message} = { alert=>'error', text=>$LANG->{error}{no_information} };
    }
    event_verify( $events{lic_valid}, sub{ $events{lic_valid}{message}{alert} eq 'notice' } );
  }

  if( $param{midnight} )
  {
    my $time = time();
    unless( $events{midnight}{check_time} )
    {
      $modified = 1;
      $events{midnight}{check_time} = get_midnight($time);
    }
  }

  close(STDERR);
  open(STDERR,'>&OLDERR');
  close(OLDERR);

  if( $param{dont_save} )
  {
    $events{modified_flag} = $modified;
  }
  else
  {
    SaveEvents(\%events) if $modified;
  }

  return \%events;
}


sub DropEventCache(%)
{
  my @names = @_;

  my $events = LoadEvents();
  return undef unless( $events );

  my $modified = undef;
  foreach (@names)
  {
    if( exists($events->{$_}) )
    {
      my $evt = $events->{$_};
      if( defined($evt) && $evt->{check_time} )
      {
        $evt->{check_time} = 0;
        $modified = 1;
      }
    }
  }

  if( $modified )
  {
    return SaveEvents($events);
  }
  else
  {
    return 1;
  }
}


1;
__END__
