#!/usr/local/ap-mailfilter3/bin/perl -w
#######################################################################
# Default paths and constants

use strict;
use POSIX qw(strftime mktime);
use Data::Dumper;
use File::Copy;
use Time::Local;

use vars qw($CONST $CONFIG @ERR);

BEGIN {
  require 'const.pl';
  require 'config.pl';
  require 'conffiles.pl';
  require 'datetime.pl';
};


sub LoadUpdaterLog($;$)
{
  my ($logpath,$loglevel) = @_;
  $loglevel = 'info' unless defined($loglevel) && $loglevel =~ /^[inwe]/i;

  my %STATUSES = (
                    'a' => 'info',
                    'i' => 'notice',
                    'n' => 'notice',
                    'w' => 'warning',
                    'e' => 'error',
                    'f' => 'error'
                 );
  $STATUSES{a} = undef if( $loglevel =~ /^[nwe]/i );
  $STATUSES{i} = $STATUSES{n} = undef if( $loglevel =~ /^[we]/i );
  $STATUSES{w} = undef if( $loglevel =~ /^e/i );

  my @log;

  unless( open( LOG, "<$logpath" ) )
  {
    push @ERR, "Can't open file $logpath: $!";
    return \@log;
  }

  my $filesize = -s LOG;
  if( $filesize > $CONST->{etc}{logsize_limit} )
  {
    # Skip file begining over limit
    seek( LOG, -$CONST->{etc}{logsize_limit}, 2 );
    my $dummy = <LOG>;
  }

  my @lines = <LOG>;
  close LOG;

  my $count = 0;
  my $time_limit = time() - $CONST->{etc}{logtime_limit};
  while(my $line = shift(@lines))
  {
    if( $line =~ /^\[(\d+)-(\d+)-(\d+)\s+(\d+):(\d+):(\d+)\s+(\S)\]\s+(.*)/ )
    {
      my ($d, $m, $y, $H, $M, $S, $status, $text) = ($1, $2, $3, $4, $5, $6, $7, $8);
      $status = $STATUSES{lc($status)};
      next unless $status;

      my $time = timelocal($S, $M, $H, $d, $m-1, $y-1900 );
      next if $time < $time_limit;

      $text =~ s/^\s+//;
      $text =~ s/\s+$//;
      $text =~ s/\s+/ /g;

      my %item = ( status=>$status, 'time'=>$time, text=>"keepup2date: $text", count=>$count++ );
      push @log, \%item;
    }
  }

  my $log_size = scalar(@log);
  splice( @log, 0, $log_size-$CONST->{etc}{logcount_limit} ) if( $log_size > $CONST->{etc}{logcount_limit} );

  return \@log;
}


sub LoadSyslog($;$)
{
  my ($logpath,$loglevel) = @_;
  $loglevel = 'info' unless defined($loglevel) && $loglevel =~ /^[inwe]/i;

  my %STATUSES = (
                    'kasinfo' => 'info',
                    'aapinfo' => 'info',
                    'kasnotice' => 'notice',
                    'aapnotice' => 'notice',
                    'kaswarning' => 'warning',
                    'aapwarning' => 'warning',
                    'kaserror' => 'error',
                    'aaperror' => 'error',
                    'kascritical' => 'error',
                    'aapcritical' => 'error',
                    'kasalert' => 'error',
                    'aapalert' => 'error',
                    'kasemerg' => 'error',
                    'aapemerg' => 'error',
                 );
  $STATUSES{kasinfo} = $STATUSES{aapinfo} = undef if( $loglevel =~ /^[nwe]/i );
  $STATUSES{kasnotice} = $STATUSES{aapnotice} = undef if( $loglevel =~ /^[we]/i );
  $STATUSES{kaswarning} = $STATUSES{aapwarning} = undef = undef if( $loglevel =~ /^e/i );

  my @log;

  unless( open( LOG, "<$logpath" ) )
  {
    push @ERR, "Can't open file $logpath: $!";
    return \@log;
  }

  my $filesize = -s LOG;
  if( $filesize > $CONST->{etc}{logsize_limit} )
  {
    # Skip file begining over limit
    seek( LOG, -$CONST->{etc}{logsize_limit}, 2 );
    my $dummy = <LOG>;
  }

  my @lines = <LOG>;
  close LOG;

  my $count = 0;
  my $time_limit = time() - $CONST->{etc}{logtime_limit};
  while(my $line = shift(@lines))
  {
    if( $line =~ /^(.+?)(\S+?\:)\s+((KAS|AAP)(INFO|NOTICE|WARNING|ERROR|((KAS)?CRITICAL)|ALERT|EMERG))\:?\s+(.*)$/i )
    {
      my ($datestr,$module,$status,$msgstr) = ($1,$2,$3,$8);
      $status = $STATUSES{lc($status)};
      next unless $status;

      my $time = ParseSyslogDate($datestr);
      next if $time < $time_limit;

      my $text = "$module $msgstr";
      $text =~ s/^\s+//;
      $text =~ s/\s+$//;
      $text =~ s/\s+/ /g;

      my %item = ( status=>$status, 'time'=>$time, text=>$text, count=>$count++ );
      push @log, \%item;
    }
  }

  my $log_size = scalar(@log);
  splice( @log, 0, $log_size-$CONST->{etc}{logcount_limit} ) if( $log_size > $CONST->{etc}{logcount_limit} );

  return \@log;
}


sub LogExists(@)
{
  my (@logname) = @_;

  return @logname unless @logname;

  my %logname;
  foreach (@logname) { $logname{$_}++; }

  my @logexist;
  foreach (keys %logname) { push @logexist, $_ if( -e $_ && -f _ ); }
  return @logexist;
}


sub FindSyslog($)
{
  my @logname;
  ##if( defined($CONFIG->{FilterUserLogFile}) && -r $CONFIG->{FilterUserLogFile} && -f _ )
  ##{
  ##  push @logname, $CONFIG->{FilterUserLogFile};
  ##  return \@logname;
  ##}

  my $syslog_conf_path = '/etc/syslog.conf';
  my %FACILITY_MAP = (
                        'auth'     => 'auth',
                        'authpriv' => 'authpriv',
                        'console'  => 'console',
                        'cron'     => 'cron',
                        'daemon'   => 'daemon',
                        'ftp'      => 'ftp',
                        'kern'     => 'kern',
                        'security' => 'security',
                        'syslog'   => 'syslog',
                        'lpr'      => 'lpr',
                        'mail'     => 'mail',
                        'news'     => 'news',
                        'user'     => 'user',
                        'uucp'     => 'uucp',
                        'local0'   => 'local0',
                        'local1'   => 'local1',
                        'local2'   => 'local2',
                        'local3'   => 'local3',
                        'local4'   => 'local4',
                        'local5'   => 'local5',
                        'local6'   => 'local6',
                        'local7'   => 'local7',
                        'mark'     => 'mark',
                     );

  my %LEVEL_ORDER =  (
                        'emerg'   => 0,
                        'alert'   => 1,
                        'crit'    => 2,
                        'err'     => 3,
                        'warning' => 4,
                        'notice'  => 5,
                        'info'    => 6,
                        'debug'   => 7
                    );
  my $levels_rx = '';
  my @levels = keys(%LEVEL_ORDER);
  foreach (@levels)
  {
    $levels_rx .= '|' if $levels_rx ne '';
    $levels_rx .= quotemeta($_);
  }

  my ($facility) = @_;
  $facility = $FACILITY_MAP{lc($facility)};
  $facility = $FACILITY_MAP{'mail'} unless defined $facility;

  unless( open( SYSLOGCONF, "<$syslog_conf_path" ) )
  {
    warn "Can't open $syslog_conf_path: $!\n";
    return undef;
  }

  my %logs = ( my_facility =>  { err=>[], warning=>[], notice=>[], info=>[] },
               all_facility => { err=>[], warning=>[], notice=>[], info=>[] } );
  my ($to_prog, $to_host);
  while( <SYSLOGCONF> )
  {
    chomp;
    next if( /^\s*$/ );

    if( /^\s*(#?)!(\+?)\*/ )
    {
      $to_prog = undef;
      next;
    }
    elsif( /^\s*(#?)!\S+/ )
    {
      $to_prog = 1;
      next;
    }
    elsif( /^\s*(#?)\+\*/ )
    {
      $to_host = undef;
      next;
    }
    elsif( /^\s*(#?)[+-]\S+/ )
    {
      $to_host = 1;
      next;
    }
    elsif( /^\s*#/ )
    {
      next ;
    }

    if( !$to_prog && !$to_host && /^\s*(\S+)\s+(.*)$/ )
    {
      my ($selectors,$action) = ($1,$2);
      if( $action =~ /^\-(\/.*)$/ ) { $action = $1; }
      next unless $action =~ /^\//;
      next if $action =~ /^\/dev\//;

      foreach ( split(/;/,$selectors) )
      {
        if( /^([^\.]+)\.([^\.]+)$/ )
        {
          my ($selfacility, $sellevel) = (lc($1),lc($2));
          if( $selfacility eq '*' || $selfacility eq $facility )
          {
            my $key = ($selfacility eq '*')? 'all_facility' : 'my_facility';
            if( $sellevel eq '*' )
            {
              push @{$logs{$key}{err}}, $action;
              push @{$logs{$key}{warning}}, $action;
              push @{$logs{$key}{notice}}, $action;
              push @{$logs{$key}{info}}, $action;
            }
            elsif( $sellevel =~ /^(>|!<=|!=<)($levels_rx)$/io )
            {
              my $level = $LEVEL_ORDER{$2};
              if( defined $level )
              {
                push @{$logs{$key}{err}}, $action     if( $level > $LEVEL_ORDER{err} );
                push @{$logs{$key}{warning}}, $action if( $level > $LEVEL_ORDER{warning} );
                push @{$logs{$key}{notice}}, $action  if( $level > $LEVEL_ORDER{notice} );
                push @{$logs{$key}{info}}, $action    if( $level > $LEVEL_ORDER{info} );
              }
            }
            elsif( $sellevel =~ /^(<|!|!=>|!>=)($levels_rx)$/io )
            {
              my $level = $LEVEL_ORDER{$2};
              if( defined $level )
              {
                push @{$logs{$key}{err}}, $action     if( $level < $LEVEL_ORDER{err} );
                push @{$logs{$key}{warning}}, $action if( $level < $LEVEL_ORDER{warning} );
                push @{$logs{$key}{notice}}, $action  if( $level < $LEVEL_ORDER{notice} );
                push @{$logs{$key}{info}}, $action    if( $level < $LEVEL_ORDER{info} );
              }
            }
            elsif( $sellevel =~ /^(<=|=<|!>)($levels_rx)$/io )
            {
              my $level = $LEVEL_ORDER{$2};
              if( defined $level )
              {
                push @{$logs{$key}{err}}, $action     if( $level <= $LEVEL_ORDER{err} );
                push @{$logs{$key}{warning}}, $action if( $level <= $LEVEL_ORDER{warning} );
                push @{$logs{$key}{notice}}, $action  if( $level <= $LEVEL_ORDER{notice} );
                push @{$logs{$key}{info}}, $action    if( $level <= $LEVEL_ORDER{info} );
              }
            }
            elsif( $sellevel =~ /^=($levels_rx)$/io )
            {
              my $level = $LEVEL_ORDER{$1};
              if( defined $level )
              {
                push @{$logs{$key}{err}}, $action     if( $level == $LEVEL_ORDER{err} );
                push @{$logs{$key}{warning}}, $action if( $level == $LEVEL_ORDER{warning} );
                push @{$logs{$key}{notice}}, $action  if( $level == $LEVEL_ORDER{notice} );
                push @{$logs{$key}{info}}, $action    if( $level == $LEVEL_ORDER{info} );
              }
            }
            elsif( $sellevel =~ /^!=($levels_rx)$/io )
            {
              my $level = $LEVEL_ORDER{$1};
              if( defined $level )
              {
                push @{$logs{$key}{err}}, $action     if( $level != $LEVEL_ORDER{err} );
                push @{$logs{$key}{warning}}, $action if( $level != $LEVEL_ORDER{warning} );
                push @{$logs{$key}{notice}}, $action  if( $level != $LEVEL_ORDER{notice} );
                push @{$logs{$key}{info}}, $action    if( $level != $LEVEL_ORDER{info} );
              }
            }
            else # if( $sellevel =~ /^(>=|=>|!<)?($levels_rx)$/io )
            {
              my $level = $LEVEL_ORDER{$sellevel};
              if( defined $level )
              {
                push @{$logs{$key}{err}}, $action     if( $level >= $LEVEL_ORDER{err} );
                push @{$logs{$key}{warning}}, $action if( $level >= $LEVEL_ORDER{warning} );
                push @{$logs{$key}{notice}}, $action  if( $level >= $LEVEL_ORDER{notice} );
                push @{$logs{$key}{info}}, $action    if( $level >= $LEVEL_ORDER{info} );
              }
            }
          }
        }
      }
    }
  }
  close(SYSLOGCONF);

  {
    my %logname;
    foreach (@{$logs{my_facility}{err}}) { $logname{$_} += 8; }
    foreach (@{$logs{my_facility}{warning}}) { $logname{$_} += 4; }
    foreach (@{$logs{my_facility}{notice}}) { $logname{$_} += 2; }
    foreach (@{$logs{my_facility}{info}}) { $logname{$_} += 1; }

    @logname = sort{ $logname{$b} <=> $logname{$a} } LogExists( keys(%logname) );
  }

  unless(@logname)
  {
    my %logname;
    foreach (@{$logs{all_facility}{err}}) { $logname{$_} += 8; }
    foreach (@{$logs{all_facility}{warning}}) { $logname{$_} += 4; }
    foreach (@{$logs{all_facility}{notice}}) { $logname{$_} += 2; }
    foreach (@{$logs{all_facility}{info}}) { $logname{$_} += 1; }

    @logname = sort{ $logname{$b} <=> $logname{$a} } LogExists( keys(%logname) );
  }

  return (@logname)? \@logname: undef;
}


sub syslogid($;$)
{
  my ($id,$count) = @_;
  return (defined($count) && $count<2)? "syslog " : "syslog$id";
}


sub Build_Stat_conf()
{
  my $stat_root = $CONFIG->{StatPath};
  my $stat_conf_path = $CONFIG->{StatConfPath};
  my $stat_conf_tmp_path = $stat_conf_path . ".$$.tmp";

  my $log_path = $CONFIG->{LogsRootPath};
  my $syslogs = FindSyslog($CONFIG->{LogFacility});
  my $syslogs_count = ( defined($syslogs) )? scalar(@{$syslogs}): 0;
  my $i;

  unless( open(CONF,">$stat_conf_tmp_path") )
  {
    warn "Can't open $stat_conf_tmp_path for writing: $!\n";
    push @ERR, "Can't open $stat_conf_tmp_path for writing: $!";
    return undef;
  }

  my $kasstatus_rx = '^KAS(INFO|NOTICE|WARNING|ERROR|((KAS)?CRITICAL)|ALERT|EMERG)\:?$';
  my $statstatus_rx = '^KASSTATS\:?$';
  my $updateproc_rx = 'sfupdate|keepup2date';
  my $licproc_rx = 'kas-license|install-key|remove-key';
  my $filtproc_rx = '^(?:(?!(('.$updateproc_rx.')|('.$licproc_rx.'))).)*$';

  print CONF qq`DatabaseDir $CONFIG->{CCStatPath}
GraphDir    $CONFIG->{CCWwwStatPath}
LockFile    $CONFIG->{CCStatPath}/lockfile.lck
#
OutputLog log_syslog      $CONFIG->{CCSyslogLogPath}
OutputLog log_updater     $CONFIG->{CCUpdaterLogPath}
OutputLog log_keepup2date $CONFIG->{CCKeepup2dateLogPath}
OutputLog log_license     $CONFIG->{CCSLicenseLogPath}
#
LogMask apmailfilter __STATS:7       5="$statstatus_rx" 6="^AAP"
`;

for( $i=1; $i<=$syslogs_count; $i++ )
{
  my $id = syslogid($i,$syslogs_count);
  print CONF qq`LogMask $id      log_updater     5="$updateproc_rx" 6="$kasstatus_rx"
LogMask $id      log_updater     6="$updateproc_rx" 7="$kasstatus_rx"
LogMask $id      log_license     5="$licproc_rx" 6="$kasstatus_rx"
LogMask $id      log_license     6="$licproc_rx" 7="$kasstatus_rx"
LogMask $id      log_syslog      5="$filtproc_rx" 6="$kasstatus_rx"
LogMask $id      log_syslog      6="$filtproc_rx" 7="$kasstatus_rx"
`;
}

print CONF qq`LogMask updater      log_keepup2date 1="^\\[" 3="^[A-Za-z]\]\$"
#
`;

for( $i=1; $i<=$syslogs_count; $i++ )
{
  my $id = syslogid($i,$syslogs_count);
  print CONF qq`DateFormat $id      1,2,3 "%b %d %T %y"
`;
}

print CONF qq`DateFormat apmailfilter 2,3   "%d-%m-%y %T"
DateFormat updater      1,2   "[%d-%m-%Y %T"
#
StatusDecoration spam          visiblename="$CONST->{detected_classes}{spam}"     fillcolor=FF6666 graphcolor=-FF6666
StatusDecoration probable_spam visiblename="$CONST->{detected_classes}{probable}" fillcolor=FFCA42 graphcolor=FFCA42
StatusDecoration formal        visiblename="$CONST->{detected_classes}{formal}"   fillcolor=6699FF graphcolor=0000ff
StatusDecoration not_detected  visiblename="$CONST->{detected_classes}{normal}"   fillcolor=1F9986 graphcolor=-1F9986
StatusDecoration blacklisted   visiblename="$CONST->{detected_classes}{black}"    fillcolor=333333 graphcolor=-010101
StatusDecoration trusted       visiblename="$CONST->{detected_classes}{trust}"    fillcolor=66CC33 graphcolor=c0c0c0
StatusDecoration __other       visiblename="$CONST->{detected_classes}{other}"    fillcolor=c0c0c0 graphcolor=808080
#
InputFile updater      $CONFIG->{UpdaterLogPath} S
InputFile apmailfilter $CONFIG->{FilterStatLogFile}
#
`;

  if( $syslogs && scalar(@{$syslogs}) )
  {
    $i=1;
    foreach (@{$syslogs})
    {
      my $id = syslogid($i++,$syslogs_count);
      print CONF "InputFile $id      $_ Z\n";
    }
    print CONF "#\n";
  }
  else
  {
    warn "No syslogs found!\n";
    push @ERR, "No syslogs found!";
  }
  close(CONF);

  unless( move $stat_conf_tmp_path, $stat_conf_path )
  {
    warn "Can't move $stat_conf_tmp_path to $stat_conf_path: $!\n";
    push @ERR, "Can't move $stat_conf_tmp_path to $stat_conf_path: $!";
    return undef;
  }
  return 1;
}


1;
__END__
